SystemOrganization addCategory: #'Comet-Core'!

!WAPlatform methodsFor: '*comet-core' stamp: 'jf 1/22/2009 02:11'!
withName: aString withPriority: anInteger fork: aBlock
	self subclassResponsibility! !

WAObject subclass: #CTHandler
	instanceVariableNames: 'id pusher socket session context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTHandler class methodsFor: 'instance creation' stamp: 'lr 8/30/2006 11:56'!
on: aPusher session: aSession context: aContext
	^ self basicNew initializeOn: aPusher session: aSession context: aContext! !

!CTHandler methodsFor: 'private' stamp: 'jf 1/23/2009 23:16'!
basicConnect: aRequestContext
	| root |
	WACurrentRequestContext use: aRequestContext during: [
		socket := aRequestContext responseStream encodingSocket.
		socket pushData: (String streamContents: [ :stream |
			root := WAHtmlRoot context: context.
			WAResponse new
				contentType: ((WAMimeType main: 'text' sub: 'html') 
					charset: aRequestContext application charSet;
					yourself);
				cacheForever;
				contents: (String streamContents: [ :contents |
					session updateRoot: root. 
					(WAHtmlDocument
						on: contents
						codec: aRequestContext codec)
							open: root.
					contents nextPutAll: (String new: 1024 withAll: Character space) ]);
				writeOn: stream ]) ]! !

!CTHandler methodsFor: 'private' stamp: 'lr 7/12/2006 14:33'!
basicDisconnect
	socket ifNil: [ ^ self ].
	socket destroy.
	socket := nil! !

!CTHandler methodsFor: 'private' stamp: 'lr 10/28/2007 14:45'!
basicTerminate
	thisContext swapSender: nil.
	Processor activeProcess terminate! !

!CTHandler methodsFor: 'actions' stamp: 'lr 11/3/2008 22:02'!
connect: aRequestContext
	self
		basicDisconnect;
		basicConnect: aRequestContext.
	self pusher register: self.
	self basicTerminate! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 7/12/2006 11:22'!
id
	^ id! !

!CTHandler methodsFor: 'initialization' stamp: 'lr 1/10/2009 12:39'!
initializeOn: aPusher session: aSession context: aContext
	id := WAKeyGenerator current
		keyOfLength: 8.
	pusher := aPusher.
	session := aSession.
	context := aContext! !

!CTHandler methodsFor: 'actions' stamp: 'lr 8/30/2006 12:11'!
push: aString
	socket ifNil: [ ^ false ].
	^ socket pushData: aString! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 7/12/2006 14:24'!
pusher
	^ pusher! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 8/30/2006 11:32'!
session
	^ session! !

WAObject subclass: #CTPusher
	instanceVariableNames: 'mutex handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

CTPusher class
	instanceVariableNames: 'process delay'!

CTPusher class
	instanceVariableNames: 'process delay'!

!CTPusher class methodsFor: 'initialization' stamp: 'jf 1/22/2009 02:43'!
initialize
	WAPlatform current
		addToStartUpList: self;
		addToShutDownList: self! !

!CTPusher class methodsFor: 'private' stamp: 'lr 8/30/2006 10:25'!
pingProcess
	self allInstancesDo: [ :each | each ping ].
	delay wait! !

!CTPusher class methodsFor: 'initialization' stamp: 'jf 1/21/2009 17:42'!
shutDown
	process ifNil: [ ^ self ].
	WAPlatform current terminateProcess: process.
	process := nil! !

!CTPusher class methodsFor: 'initialization' stamp: 'jf 1/22/2009 02:43'!
startUp
	self shutDown.
	delay := Delay forSeconds: 15.
	process := WAPlatform current
		withName: 'ping' withPriority: 20
		fork: [ [ self pingProcess ] repeat ]! !

!CTPusher class methodsFor: 'initialization' stamp: 'jf 1/22/2009 02:43'!
unload
	WAPlatform current
		removeFromStartUpList: self;
		removeFromShutDownList: self.
	self shutDown! !

!CTPusher methodsFor: 'accessing' stamp: 'jf 1/23/2009 11:53'!
canvas
	"Answer a fake rendering canvas for this pusher."
	
	^ WARenderCanvas 
		context: WARenderContext new! !

!CTPusher methodsFor: 'accessing' stamp: 'lr 7/12/2006 11:50'!
handlers
	"Answer the current list of handlers."

	^ handlers! !

!CTPusher methodsFor: 'initialization' stamp: 'jf 1/22/2009 02:43'!
initialize
	super initialize.
	mutex := WAPlatform current semaphoreClass forMutualExclusion.
	handlers := Array new! !

!CTPusher methodsFor: 'public' stamp: 'lr 10/14/2008 00:51'!
javascript: aBlock
	"Evaluate aBlock with an SUScript instance and pushes the resulting script-string aString to all associated handlers."
	
	| script |
	aBlock value: (script := JSScript on: self canvas).
	self push: (String streamContents: [ :stream |
		stream
			nextPutAll: '<script type="text/javascript">parent.Comet.eval(';
			javascript: script asJavascript;
			nextPutAll: ')</script>' ])! !

!CTPusher methodsFor: 'accessing' stamp: 'lr 7/12/2006 15:01'!
mutex
	"Answer the mutex in use with the reciever."

	^ mutex! !

!CTPusher methodsFor: 'protected' stamp: 'lr 8/30/2006 11:15'!
ping
	"Ping all the associated handlers."

	self mutex critical: [ handlers := handlers select: [ :each | each push: ' ' ] ]! !

!CTPusher methodsFor: 'protected' stamp: 'mb 10/26/2006 12:10'!
push: aString
	"Push aString to all associated handlers."

	handlers do: [ :each | each push: aString ]! !

!CTPusher methodsFor: 'protected' stamp: 'lr 8/30/2006 10:43'!
register: aHandler
	"Register a new handler into this pusher."

	mutex critical: [ handlers := handlers copyWith: aHandler ]! !

WARequestFilter subclass: #CTFilter
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTFilter methodsFor: 'processing' stamp: 'lr 11/3/2008 22:02'!
cometForContext: aRequestContext
	^ (handlers
		at: (aRequestContext request fields
			at: CTComet headerField
			ifAbsent: [ ^ nil ])
		ifAbsent: [ ^ nil ])
			connect: aRequestContext! !

!CTFilter methodsFor: 'processing' stamp: 'lr 1/19/2009 19:23'!
handleFiltered: aRequestContext
	aRequestContext stream
		ifNil: [ self error: 'Streaming-server required.' ].
	(aRequestContext request includesKey: CTComet headerField)
		ifTrue: [ self cometForContext: aRequestContext ]
		ifFalse: [ super handleFiltered: aRequestContext ]! !

!CTFilter methodsFor: 'initialization' stamp: 'lr 11/3/2008 20:43'!
initialize
	super initialize.
	handlers := Dictionary new! !

!CTFilter methodsFor: 'testing' stamp: 'lr 11/3/2008 20:50'!
isCometFilter
	^ true! !

!CTFilter methodsFor: 'public' stamp: 'lr 11/3/2008 20:59'!
registerPusher: aPusher context: aContext
	| handler |
	handler := CTHandler on: aPusher session: self handler context: aContext.
	handlers at: handler id put: handler.
	^ handler id! !

!WARequestFilter methodsFor: '*comet-core' stamp: 'lr 11/3/2008 20:50'!
isCometFilter
	^ false! !

SUPrototype subclass: #CTComet
	instanceVariableNames: 'pusher url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTComet class methodsFor: 'configuration' stamp: 'lr 7/12/2006 12:10'!
headerField
	^ '_c'! !

!CTComet class methodsFor: 'configuration' stamp: 'lr 7/12/2006 12:10'!
terminator
	^ Character value: 0! !

!CTComet methodsFor: 'actions' stamp: 'lr 7/3/2006 13:54'!
connect
	self call: 'connect'! !

!CTComet methodsFor: 'private' stamp: 'lr 5/28/2006 22:00'!
defaultArguments
	^ (Array with: self url) , super defaultArguments! !

!CTComet methodsFor: 'private' stamp: 'lr 5/28/2006 22:07'!
defaultPrototype
	^ 'Comet'! !

!CTComet methodsFor: 'private' stamp: 'lr 7/12/2006 12:10'!
defaultUrl
	^ self canvas context actionUrl
		addParameter: self class headerField
		value: self pusher id! !

!CTComet methodsFor: 'actions' stamp: 'lr 7/3/2006 13:54'!
disconnect
	self call: 'disconnect'! !

!CTComet methodsFor: 'accessing-readonly' stamp: 'lr 11/3/2008 22:01'!
filter
	^ canvas session filters
		detect: [ :each | each isCometFilter ]
		ifNone: [ canvas session addFilterFirst: CTFilter new ]! !

!CTComet methodsFor: 'initialization' stamp: 'lr 11/3/2008 20:53'!
initialize
	super initialize.
	self create! !

!CTComet methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:44'!
pusher
	^ pusher! !

!CTComet methodsFor: 'accessing' stamp: 'lr 11/3/2008 20:52'!
pusher: aPusher
	self url: (self canvas context actionUrl
		withParameter: self class headerField
		value: (self filter
			registerPusher: (pusher := aPusher)
			context: self canvas context))! !

!CTComet methodsFor: 'options' stamp: 'lr 7/3/2006 13:37'!
reconnect: aBoolean
	self optionAt: 'reconnect' put: aBoolean! !

!CTComet methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:33'!
url
	^ url ifNil: [ url := self defaultUrl ]! !

!CTComet methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:34'!
url: anUrl
	url := anUrl! !

!WAPackage class methodsFor: '*comet-core' stamp: 'jf 1/22/2009 03:22'!
cometCore
	^ self new
		name: 'Comet-Core';
		addDependency: 'Seaside-Canvas';
		addDependency: 'Scriptaculous-Core';
		yourself! !

!WARenderCanvas methodsFor: '*comet-core' stamp: 'lr 7/12/2006 12:17'!
comet
	^ CTComet on: self! !

WAFileLibrary subclass: #CTLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTLibrary methodsFor: 'scripts' stamp: 'lr 11/3/2008 21:00'!
cometJs
	^ '/* Seaside Comet API
 *  (c) 2006-2008 Lukas Renggli (renggli@gmail.com)
 */

Comet = Class.create();
Comet.prototype = {
	initialize: function (url, options) {
		this.url = url;
		this.options = Object.extend({
			reconnect: true
		}, options || {});
		this.iframe = this.newIframe();
		document.body.appendChild(this.iframe);
	},
	connect: function() {
		this.iframe.onLoad = this.onLoad.bind(this);
		this.iframe.src = this.url;
	},
	disconnect: function() {
		this.iframe.onLoad = Prototype.emptyFunction; 
		this.iframe.src = null;
	},
	onLoad: function() {
		this.disconnect();
		if (this.options.reconnect)
			this.connect();
	},
	newIframe: function() {
		var frame = document.createElement("iframe");
		frame.style.display = "block";
		frame.style.visibility = frame.style.overflow = "hidden";
		frame.style.width = frame.style.height = frame.style.border = frame.style.margin = frame.style.padding = "0";
		return frame;
	}
};
Comet.eval = function(code) { return eval(code); };'! !

!CTLibrary methodsFor: 'accessing' stamp: 'lr 1/2/2008 20:11'!
selectorsToInclude
	^ #( cometJs )! !
CTPusher initialize!
